@@ -10,7 +10,7 @@ try:
     from gnuradio import gr  # pytype: disable=import-error
     from gnuradio.eng_arg import eng_float  # pytype: disable=import-error
     from gnuradio.eng_arg import intx  # pytype: disable=import-error
-except ModuleNotFoundError:
+except ModuleNotFoundError:  # pragma: no cover
     print(
         "Run from outside a supported environment, please run via Docker (https://github.com/IQTLabs/gamutRF#readme)"
     )
@@ -20,7 +20,6 @@ from prometheus_client import Gauge
 from prometheus_client import start_http_server
 
 from gamutrf.grscan import grscan
-from gamutrf.utils import ROLLOVERHZ
 
 
 def init_prom_vars():
@@ -89,15 +88,15 @@ def argument_parser():
         "--logaddr",
         dest="logaddr",
         type=str,
-        default="127.0.0.1",
-        help="Log UDP results to this address",
+        default="0.0.0.0",  # nosec
+        help="Log FFT results to this address",
     )
     parser.add_argument(
         "--logport",
         dest="logport",
         type=int,
         default=8001,
-        help="Log UDP results to this port",
+        help="Log FFT results to this port",
     )
     parser.add_argument(
         "--promport",
@@ -140,16 +139,14 @@ def main():
         print("Error: freq_start must be less than freq_end")
         sys.exit(1)
 
-    if options.freq_end - options.freq_start <= ROLLOVERHZ:
-        print(
-            f"Difference between freq_end and freq_start must be greater than {ROLLOVERHZ}"
-        )
-        sys.exit(1)
-
     if options.freq_end > 6e9:
         print("Error: freq_end must be less than 6GHz")
         sys.exit(1)
 
+    if options.freq_start < 70e6:
+        print("Error: freq_start must be at least 70MHz")
+        sys.exit(1)
+
     prom_vars = init_prom_vars()
     prom_vars["freq_start_hz"].set(options.freq_start)
     prom_vars["freq_end_hz"].set(options.freq_end)
