@@ -7,16 +7,20 @@ import time
 from urllib.parse import urlparse
 
 import sigmf
+import numpy as np
+import matplotlib
+import matplotlib.pyplot as plt
 
 from gamutrf.sigwindows import freq_excluded
 from gamutrf.sigwindows import parse_freq_excluded
-from gamutrf.utils import ETTUS_ANT
-from gamutrf.utils import ETTUS_ARGS
+from gamutrf.utils import ETTUS_ANT, ETTUS_ARGS, MPL_BACKEND, WIDTH, HEIGHT, DPI
 
 
+NFFT = int(os.getenv("NFFT", "0"))
 SAMPLE_TYPE = "s16"
 MIN_SAMPLE_RATE = int(1e6)
 MAX_SAMPLE_RATE = int(30 * 1e6)
+FFT_FILE = "/dev/shm/fft.dat"  # nosec
 
 
 class SDRRecorder:
@@ -72,6 +76,37 @@ class SDRRecorder:
             os.rename(dotfile, sample_file)
         return record_status
 
+    @staticmethod
+    def fft_spectrogram(
+        sample_file, fft_file, sample_count, sample_rate, center_freq, nfft
+    ):
+        if os.path.exists(fft_file):
+            matplotlib.use(MPL_BACKEND)
+            png_file = sample_file.replace(".zst", ".png")
+            logging.info("generating spectrogram: %s", png_file)
+            i = np.memmap(fft_file, dtype=np.float32, mode="r+")
+            i = np.roll(i.reshape(-1, nfft).swapaxes(0, 1), int(nfft / 2), 0)
+            fig = plt.figure()
+            fig.set_size_inches(WIDTH, HEIGHT)
+            axes = fig.add_subplot(111)
+            axes.set_xlabel("time (s)")
+            axes.set_ylabel("freq (MHz)")
+            fc = center_freq / 1e6
+            fo = sample_rate / 1e6 / 2
+            extent = (0, sample_count / sample_rate, fc - fo, fc + fo)
+            im = axes.imshow(i, cmap="turbo", origin="lower", extent=extent)
+            axes.axis("auto")
+            axes.minorticks_on()
+            plt.colorbar(im, ax=axes)
+            plt.sci(im)
+            plt.savefig(png_file, dpi=DPI)
+            axes.images.remove(im)
+            fig.clear()
+            plt.close()
+            plt.cla()
+            plt.clf()
+            os.remove(fft_file)
+
     def run_recording(
         self,
         path,
@@ -95,6 +130,11 @@ class SDRRecorder:
             record_status = self.write_recording(
                 sample_file, sample_rate, sample_count, center_freq, gain, agc, rxb
             )
+            if NFFT:
+                self.fft_spectrogram(
+                    sample_file, FFT_FILE, sample_count, sample_rate, center_freq, NFFT
+                )
+
             if sigmf_:
                 meta = sigmf.SigMFFile(
                     data_file=sample_file,
@@ -130,7 +170,8 @@ class EttusRecorder(SDRRecorder):
     ):
         # Ettus "nsamps" API has an internal limit, so translate "stream for druation".
         duration = round(sample_count / sample_rate)
-        return [
+        rxb = min(rxb, sample_rate)
+        args = [
             "/usr/local/bin/mt_rx_samples_to_file",
             "--file",
             sample_file,
@@ -151,6 +192,20 @@ class EttusRecorder(SDRRecorder):
             "--spb",
             str(rxb),
         ]
+        if NFFT:
+            args.extend(
+                [
+                    "--nfft",
+                    str(NFFT),
+                    "--nfft_overlap",
+                    str(int(NFFT / 8)),
+                    "--nfft_ds",
+                    str(int(1)),
+                    "--fft_file",
+                    FFT_FILE,
+                ]
+            )
+        return args
 
     def write_recording(
         self, sample_file, sample_rate, sample_count, center_freq, gain, agc, rxb
